@@ -37,6 +37,7 @@
 #include <eel/eel-debug.h>
 #include <libgnome/gnome-util.h>
 #include <glib/gi18n.h>
+#include <glib/gstrfuncs.h>
 #include <gio/gio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -51,7 +52,7 @@
 static void update_xdg_dir_cache (void);
 static void schedule_user_dirs_changed (void);
 static void desktop_dir_changed (void);
-
+static GFile *nautilus_find_file_insensitive_next (GFile *parent, const gchar *name);
 
 char *
 nautilus_compute_title_for_location (GFile *location)
@@ -365,7 +366,7 @@ update_xdg_dir_cache (void)
 		config_file = g_build_filename (g_get_user_config_dir (),
 						     "user-dirs.dirs", NULL);
 		file = g_file_new_for_path (config_file);
-		cached_xdg_dirs_monitor = g_file_monitor_file (file, 0, NULL);
+		cached_xdg_dirs_monitor = g_file_monitor_file (file, 0, NULL, NULL);
 		g_signal_connect (cached_xdg_dirs_monitor, "changed",
 				  G_CALLBACK (xdg_dir_cache_changed_cb), NULL);
 		g_object_unref (file);
@@ -844,6 +845,117 @@ nautilus_find_existing_uri_in_hierarchy (GFile *location)
 	return location;
 }
 
+/**
+ * nautilus_find_file_insensitive
+ * 
+ * Attempt to find a file case-insentively. If the path can be found, the
+ * returned file maps directly to it. Otherwise, a file using the
+ * originally-cased path is returned. This function performs might perform
+ * I/O.
+ * 
+ * Return value: a #GFile to a child specified by @name.
+ **/
+GFile *
+nautilus_find_file_insensitive (GFile *parent, const gchar *name)
+{
+	gchar **split_path;
+	gchar *component;
+	GFile *file, *next;
+	gint i;
+	
+	split_path = g_strsplit (name, G_DIR_SEPARATOR_S, -1);
+	
+	file = g_object_ref (parent);
+	
+	for (i = 0; (component = split_path[i]) != NULL; i++) {
+		if (!(next = nautilus_find_file_insensitive_next (file,
+		                                                  component))) {
+			/* File does not exist */
+			g_object_unref (file);
+			file = NULL;
+			break;
+		}
+		g_object_unref (file);
+		file = next;
+	}
+	g_strfreev (split_path);
+	
+	if (file) {
+		return file;
+	}
+	return g_file_get_child (parent, name);
+}
+
+static GFile *
+nautilus_find_file_insensitive_next (GFile *parent, const gchar *name)
+{
+	GFileEnumerator *children;
+	GFileInfo *info;
+	gboolean use_utf8, found;
+	char *filename, *case_folded_name, *utf8_collation_key, *ascii_collation_key, *child_key;
+	GFile *file;
+	const char *child_name, *compare_key;
+
+	/* First check the given version */
+	file = g_file_get_child (parent, name);
+	if (g_file_query_exists (file, NULL)) {
+		return file;
+	}
+	g_object_unref (file);
+	
+	ascii_collation_key = g_ascii_strdown (name, -1);
+	use_utf8 = g_utf8_validate (name, -1, NULL);
+	utf8_collation_key = NULL;	
+	if (use_utf8) {
+		case_folded_name = g_utf8_casefold (name, -1);
+		utf8_collation_key = g_utf8_collate_key (case_folded_name, -1);
+		g_free (case_folded_name);
+	}
+
+	/* Enumerate and compare insensitive */
+	filename = NULL;
+	children = g_file_enumerate_children (parent,
+	                                      G_FILE_ATTRIBUTE_STANDARD_NAME,
+	                                      0, NULL, NULL);
+	if (children != NULL) {
+		while ((info = g_file_enumerator_next_file (children, NULL, NULL))) {
+			child_name = g_file_info_get_name (info);
+			
+			if (use_utf8 && g_utf8_validate (child_name, -1, NULL)) {
+				gchar *case_folded;
+				
+				case_folded = g_utf8_casefold (child_name, -1);
+				child_key = g_utf8_collate_key (case_folded, -1);
+				g_free (case_folded);
+				compare_key = utf8_collation_key;
+			} else {
+				child_key = g_ascii_strdown (child_name, -1);
+				compare_key = ascii_collation_key;
+			}
+			
+			found = strcmp (child_key, compare_key) == 0;
+			g_free (child_key);
+			if (found) {
+				filename = g_strdup (child_name);
+				break;
+			}
+		}
+		g_file_enumerator_close (children, NULL, NULL);
+		g_object_unref (children);
+	}
+	
+	g_free (ascii_collation_key);
+	g_free (utf8_collation_key);
+	
+	if (filename) {
+		file = g_file_get_child (parent, filename);
+		g_free (filename);
+		return file;
+	}
+	
+	return NULL;
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 void
