@@ -28,13 +28,18 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
+#include <libgnomeui/gnome-help.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
+#include <gtk/gtkcelllayout.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtktable.h>
+#include <gtk/gtktreemodel.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkliststore.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkcombobox.h>
 #include "nautilus-location-entry.h"
@@ -47,6 +52,11 @@
  * - folder browse function
  */
 
+/* TODO gio port:
+ * - see FIXME here
+ * - see FIXME in nautilus-connect-server-dialog-main.c
+ */
+
 struct _NautilusConnectServerDialogDetails {
 	NautilusApplication *application;
 	
@@ -61,6 +71,7 @@ struct _NautilusConnectServerDialogDetails {
 	GtkWidget *domain_entry;
 	GtkWidget *user_entry;
 
+	GtkWidget *bookmark_check;
 	GtkWidget *name_entry;
 };
 
@@ -71,21 +82,67 @@ EEL_CLASS_BOILERPLATE (NautilusConnectServerDialog,
 		       nautilus_connect_server_dialog,
 		       GTK_TYPE_DIALOG)
 enum {
-	RESPONSE_BROWSE,
 	RESPONSE_CONNECT
 };	
 
-/* Keep this order in sync with strings below */
-enum {
-	TYPE_SSH,
-	TYPE_ANON_FTP,
-	TYPE_FTP,
-	TYPE_SMB,
-	TYPE_DAV,
-	TYPE_DAVS,
-	TYPE_URI
+struct MethodInfo {
+	const char *scheme;
+	guint flags;
 };
 
+/* A collection of flags for MethodInfo.flags */
+enum {
+	DEFAULT_METHOD = 0x00000001,
+	
+	/* Widgets to display in setup_for_type */
+	SHOW_SHARE     = 0x00000010,
+	SHOW_PORT      = 0x00000020,
+	SHOW_USER      = 0x00000040,
+	SHOW_DOMAIN    = 0x00000080,
+	
+	IS_ANONYMOUS   = 0x00001000
+};
+
+/* Remember to fill in descriptions below */
+static struct MethodInfo methods[] = {
+	/* FIXME: we need to alias ssh to sftp */
+	{ "sftp",  SHOW_PORT | SHOW_USER },
+	{ "ftp",  SHOW_PORT | SHOW_USER },
+	{ "ftp",  DEFAULT_METHOD | IS_ANONYMOUS | SHOW_PORT},
+	{ "smb",  SHOW_SHARE | SHOW_USER | SHOW_DOMAIN },
+	{ "dav",  SHOW_PORT | SHOW_USER },
+	/* FIXME: hrm, shouldn't it work? */
+	{ "davs", SHOW_PORT | SHOW_USER },
+	{ NULL,   0 }, /* Custom URI method */
+};
+
+/* To get around non constant gettext strings */
+static const char*
+get_method_description (struct MethodInfo *meth)
+{
+	if (!meth->scheme) {
+		return _("Custom Location");
+	} else if (strcmp (meth->scheme, "sftp") == 0) {
+		return _("SSH");
+	} else if (strcmp (meth->scheme, "ftp") == 0) {
+		if (meth->flags & IS_ANONYMOUS) {
+			return _("Public FTP");
+		} else {
+			return _("FTP (with login)");
+		}
+	} else if (strcmp (meth->scheme, "smb") == 0) {
+		return _("Windows share");
+	} else if (strcmp (meth->scheme, "dav") == 0) {
+		return _("WebDAV (HTTP)");
+	} else if (strcmp (meth->scheme, "davs") == 0) {
+		return _("Secure WebDAV (HTTPS)");
+	
+	/* No descriptive text */
+	} else {
+		return meth->scheme;
+	}
+}
+
 static void
 nautilus_connect_server_dialog_finalize (GObject *object)
 {
@@ -100,6 +157,7 @@ nautilus_connect_server_dialog_finalize (GObject *object)
 	g_object_unref (dialog->details->folder_entry);
 	g_object_unref (dialog->details->domain_entry);
 	g_object_unref (dialog->details->user_entry);
+	g_object_unref (dialog->details->bookmark_check);
 	g_object_unref (dialog->details->name_entry);
 	
 	g_free (dialog->details);
@@ -120,49 +178,36 @@ nautilus_connect_server_dialog_destroy (GtkObject *object)
 static void
 connect_to_server (NautilusConnectServerDialog *dialog)
 {
+	struct MethodInfo *meth;
 	char *uri;
-	char *user_uri;
-	GnomeVFSURI *vfs_uri;
-	char *error_message;
-	char *name;
-	char *icon;
-	int type;
+	GFile *location;
+	int index;
+	GtkTreeIter iter;
 	
-	type = gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->details->type_combo));
+	/* Get our method info */
+	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->details->type_combo), &iter);
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->details->type_combo)),
+			    &iter, 0, &index, -1);
+	g_assert (index < G_N_ELEMENTS (methods) && index >= 0);
+	meth = &(methods[index]);
 
-	if (type == TYPE_URI) {
-		user_uri = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->uri_entry), 0, -1);
-		uri = gnome_vfs_make_uri_from_input (user_uri);
-		g_free (user_uri);
-	
-		vfs_uri = gnome_vfs_uri_new (uri);
-		
-		if (vfs_uri == NULL) {
-			error_message = g_strdup_printf
-				(_("\"%s\" is not a valid location."),
-				 uri);
-			eel_show_error_dialog (error_message, _("Please check the spelling and try again."), 
-					       _("Can't Connect to Server"), GTK_WINDOW (dialog));
-			g_free (error_message);
-			return;
-		} else {
-			gnome_vfs_uri_unref (vfs_uri);
-		}
+	if (meth->scheme == NULL) {
+		uri = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->uri_entry), 0, -1);
+		/* FIXME: we should validate it in some way? */
 	} else {
-		char *method, *user, *port, *initial_path, *server, *folder ,*domain ;
+		char *user, *port, *initial_path, *server, *folder ,*domain ;
 		char *t, *join;
 		gboolean free_initial_path, free_user, free_domain, free_port;
 
 		server = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->server_entry), 0, -1);
 		if (strlen (server) == 0) {
-			eel_show_error_dialog (_("You must enter a name for the server."), 
+			eel_show_error_dialog (_("Can't Connect to Server. You must enter a name for the server."), 
 					       _("Please enter a name and try again."), 
-					       _("Can't Connect to Server"), GTK_WINDOW (dialog));
+					       GTK_WINDOW (dialog));
 			g_free (server);
 			return;
 		}
 		
-		method = "";
 		user = "";
 		port = "";
 		initial_path = "";
@@ -171,30 +216,17 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 		free_user = FALSE;
 		free_domain = FALSE;
 		free_port = FALSE;
-		switch (type) {
-		case TYPE_SSH:
-			method = "sftp";
-			break;
-		case TYPE_ANON_FTP:
-			method = "ftp";
+		
+		/* FTP special case */
+		if (meth->flags & IS_ANONYMOUS) {
 			user = "anonymous";
-			break;
-		case TYPE_FTP:
-			method = "ftp";
-			break;
-		case TYPE_SMB:
-			method = "smb";
+		
+		/* SMB special case */
+		} else if (strcmp (meth->scheme, "smb") == 0) {
 			t = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->share_entry), 0, -1);
 			initial_path = g_strconcat ("/", t, NULL);
 			free_initial_path = TRUE;
 			g_free (t);
-			break;
-		case TYPE_DAV:
-			method = "dav";
-			break;
-		case TYPE_DAVS:
-			method = "davs";
-			break;
 		}
 
 		if (dialog->details->port_entry->parent != NULL) {
@@ -207,19 +239,25 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 			
 			t = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->user_entry), 0, -1);
 
-			user = gnome_vfs_escape_string (t);
+			user = g_uri_escape_string (t, G_URI_RESERVED_CHARS_ALLOWED_IN_USERINFO, FALSE);
 
 			g_free (t);
 		}
 		if (dialog->details->domain_entry->parent != NULL) {
-	
 			free_domain = TRUE;
 
 			domain = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->domain_entry), 0, -1);
 			
 			if (strlen (domain) != 0) {
-				g_free (user);
-				user = g_strconcat (domain , ";" , user, NULL);
+				t = user;
+
+				user = g_strconcat (domain , ";" , t, NULL);
+
+				if (free_user) {
+					g_free (t);
+				}
+
+				free_user = TRUE;
 			}
 		}
 
@@ -235,11 +273,11 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 		g_free (t);
 
 		t = folder;
-		folder = gnome_vfs_escape_path_string (t);
+		folder = g_uri_escape_string (t, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
 		g_free (t);
 
 		uri = g_strdup_printf ("%s://%s%s%s%s%s%s",
-				       method,
+				       meth->scheme,
 				       user, (user[0] != 0) ? "@" : "",
 				       server,
 				       (port[0] != 0) ? ":" : "", port,
@@ -260,63 +298,33 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 			g_free (domain);
 		}
 	}
-	
-	name = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->name_entry), 0, -1);
-	if (strlen (name) == 0) {
-		const char *host, *path;
-		char *path_utf8, *basename;
-		
-		g_free (name);
-		
-		vfs_uri = gnome_vfs_uri_new (uri);
-	
-		if (vfs_uri == NULL) {
-			g_warning ("Illegal uri in connect to server!\n");
-			g_free (uri);
-			g_free (name);
-			return;
-		} 
 
-		host = gnome_vfs_uri_get_host_name (vfs_uri);
-		path = gnome_vfs_uri_get_path (vfs_uri);
-		if (path != NULL &&
-		    strlen (path) > 0 &&
-		    strcmp (path, "/") != 0) {
-			path_utf8 = eel_format_uri_for_display (uri);
-			basename = g_path_get_basename (path_utf8);
-			name = g_strdup_printf (_("%s on %s"), basename, host);
-			g_free (path_utf8);
-			g_free (basename);
-		} else {
-			name = g_strdup (host);
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-	}
-		
-	if (g_str_has_prefix (uri, "smb:")) {
-		icon = "gnome-fs-smb";
-	} else if (g_str_has_prefix (uri, "ssh:") ||
-		   g_str_has_prefix (uri, "sftp:")) {
-		icon = "gnome-fs-ssh";
-	} else if (g_str_has_prefix (uri, "ftp:")) {
-		icon = "gnome-fs-ftp";
-	} else {
-		icon = "gnome-fs-share";
-	}
-	
-	gnome_vfs_connect_to_server (uri, name, icon);
 	gtk_widget_hide (GTK_WIDGET (dialog));
+	
+	location = g_file_new_for_uri (uri);
+	g_free (uri);
 
-	if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
-		nautilus_connect_server_dialog_present_uri (dialog->details->application,
-							    uri,
-							    GTK_WIDGET (dialog));
+	/* FIXME: sensitivity */
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->details->bookmark_check))) {
+		char *name;
+		NautilusBookmark *bookmark;
+		NautilusBookmarkList *list;
+
+		name = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->name_entry), 0, -1);
+		bookmark = nautilus_bookmark_new (location, strlen (name) ? name : NULL);
+		list = nautilus_bookmark_list_new ();
+		if (!nautilus_bookmark_list_contains (list, bookmark)) {
+			nautilus_bookmark_list_append (list, bookmark);
+		}
+
+		g_object_unref (bookmark);
+		g_object_unref (list);
+		g_free (name);
 	}
 
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-	g_free (uri);
-	g_free (name);
+	nautilus_connect_server_dialog_present_uri (dialog->details->application,
+						    location,
+						    GTK_WIDGET (dialog));
 }
 
 static void
@@ -324,13 +332,9 @@ response_callback (NautilusConnectServerDialog *dialog,
 		   int response_id,
 		   gpointer data)
 {
+	GError *error;
+
 	switch (response_id) {
-	case RESPONSE_BROWSE:
-		nautilus_connect_server_dialog_present_uri (dialog->details->application,
-							    "network:///",
-							    GTK_WIDGET (dialog));
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		break;
 	case RESPONSE_CONNECT:
 		connect_to_server (dialog);
 		break;
@@ -339,6 +343,18 @@ response_callback (NautilusConnectServerDialog *dialog,
 	case GTK_RESPONSE_CANCEL:
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
+	case GTK_RESPONSE_HELP :
+		error = NULL;
+		gnome_help_display_desktop_on_screen (NULL, "user-guide", "user-guide.xml",
+						      "nautilus-server-connect",
+						      gtk_window_get_screen (GTK_WINDOW (dialog)),
+						      &error);
+		if (error) {
+			eel_show_error_dialog (_("There was an error displaying help."), error->message,
+					       GTK_WINDOW (dialog));
+			g_error_free (error);
+		}
+		break;
 	default :
 		g_assert_not_reached ();
 	}
@@ -360,11 +376,17 @@ nautilus_connect_server_dialog_class_init (NautilusConnectServerDialogClass *cla
 static void
 setup_for_type (NautilusConnectServerDialog *dialog)
 {
-	int type, i;
-	gboolean show_share, show_port, show_user, show_domain;
+	struct MethodInfo *meth;
+	int index, i;
 	GtkWidget *label, *table;
-
-	type = gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->details->type_combo));
+	GtkTreeIter iter;
+	
+	/* Get our method info */
+	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->details->type_combo), &iter);
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->details->type_combo)),
+			    &iter, 0, &index, -1);
+	g_assert (index < G_N_ELEMENTS (methods) && index >= 0);
+	meth = &(methods[index]);
 
 	if (dialog->details->uri_entry->parent != NULL) {
 		gtk_container_remove (GTK_CONTAINER (dialog->details->table),
@@ -394,6 +416,10 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 		gtk_container_remove (GTK_CONTAINER (dialog->details->table),
 				      dialog->details->domain_entry);
 	}
+	if (dialog->details->bookmark_check->parent != NULL) {
+		gtk_container_remove (GTK_CONTAINER (dialog->details->table),
+				      dialog->details->bookmark_check);
+	}
 	if (dialog->details->name_entry->parent != NULL) {
 		gtk_container_remove (GTK_CONTAINER (dialog->details->table),
 				      dialog->details->name_entry);
@@ -406,7 +432,7 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 	i = 1;
 	table = dialog->details->table;
 	
-	if (type == TYPE_URI) {
+	if (meth->scheme == NULL) {
 		label = gtk_label_new_with_mnemonic (_("_Location (URI):"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_widget_show (label);
@@ -429,31 +455,6 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 		goto connection_name;
 	}
 	
-	switch (type) {
-	default:
-	case TYPE_SSH:
-	case TYPE_FTP:
-	case TYPE_DAV:
-	case TYPE_DAVS:
-		show_share = FALSE;
-		show_port = TRUE;
-		show_user = TRUE;
-		show_domain = FALSE;
-		break;
-	case TYPE_ANON_FTP:
-		show_share = FALSE;
-		show_port = TRUE;
-		show_user = FALSE;
-		show_domain = FALSE;
-		break;
-	case TYPE_SMB:
-		show_share = TRUE;
-		show_port = FALSE;
-		show_user = TRUE;
-		show_domain =TRUE;
-		break;
-	}
-
 	label = gtk_label_new_with_mnemonic (_("_Server:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
@@ -484,7 +485,7 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 
 	i++;
 	
-	if (show_share) {
+	if (meth->flags & SHOW_SHARE) {
 		label = gtk_label_new_with_mnemonic (_("_Share:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_widget_show (label);
@@ -505,7 +506,7 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 		i++;
 	}
 
-	if (show_port) {
+	if (meth->flags & SHOW_PORT) {
 		label = gtk_label_new_with_mnemonic (_("_Port:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_widget_show (label);
@@ -545,7 +546,7 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 
 	i++;
 
-	if (show_user) {
+	if (meth->flags & SHOW_USER) {
 		label = gtk_label_new_with_mnemonic (_("_User Name:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_widget_show (label);
@@ -566,7 +567,7 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 		i++;
 	}
 
-	if (show_domain) {
+	if (meth->flags & SHOW_DOMAIN) {
 		label = gtk_label_new_with_mnemonic (_("_Domain Name:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_widget_show (label);
@@ -591,7 +592,15 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 
  connection_name:
 	
-	label = gtk_label_new_with_mnemonic (_("_Name to use for connection:"));
+	gtk_widget_show (dialog->details->bookmark_check);
+	gtk_table_attach (GTK_TABLE (table), dialog->details->bookmark_check,
+			  0, 1,
+			  i, i+1,
+			  GTK_FILL, GTK_FILL,
+			  0, 0);
+	i++;
+
+	label = gtk_label_new_with_mnemonic (_("Bookmark _name:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
 	gtk_table_attach (GTK_TABLE (table), label,
@@ -612,6 +621,128 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 
 }
 
+static void
+display_server_location (NautilusConnectServerDialog *dialog, GFile *location)
+{
+#if 0 /*FIXME */
+	struct MethodInfo *meth = NULL;
+	char *scheme;
+	int i, index = 0;
+	char *folder;
+	const char *t;
+
+	/* Find an appropriate method */
+	scheme = g_file_get_uri_scheme (location);
+	g_return_if_fail (scheme != NULL);
+	
+	for (i = 0; i < G_N_ELEMENTS (methods); i++) {
+		
+		/* The default is 'Custom URI' */
+		if (methods[i].scheme == NULL) {
+			meth = &(methods[i]);
+			index = i;
+			
+		} else if (strcmp (methods[i].scheme, scheme) == 0) {
+			
+			/* FTP Special case: If no user keep searching for public ftp */
+			if (strcmp (scheme, "ftp") == 0) {
+				t = gnome_vfs_uri_get_user_name (uri);
+				if ((!t || !t[0] || strcmp (t, "anonymous") == 0) && 
+				    (!(methods[i].flags & IS_ANONYMOUS))) {
+					continue;
+				}
+			}
+			
+			meth = &(methods[i]);
+			index = i;
+			break;
+		}
+	}
+
+	g_free (scheme);
+	g_assert (meth);
+	
+	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->details->type_combo), index);
+	setup_for_type (dialog);
+	
+	/* Custom URI */
+	if (meth->scheme == NULL) {
+		gchar *uri;
+
+		/* FIXME: with gnome-vfs, we had GNOME_VFS_URI_HIDE_PASSWORD |
+		 * GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER */
+		uri = g_file_get_uri (location)
+		gtk_entry_set_text (GTK_ENTRY (dialog->details->uri_entry), uri);
+		g_free (uri);
+	
+	} else {
+		
+		folder = g_file_get_path (location);
+		if (!folder) {
+			folder = "";
+		} else if (folder[0] == '/') {
+			folder++;
+		}
+		
+		/* Server */
+		t = gnome_vfs_uri_get_host_name (uri);
+		gtk_entry_set_text (GTK_ENTRY (dialog->details->server_entry), 
+				    t ? t : "");
+		
+		/* Share */
+		if (meth->flags & SHOW_SHARE) {
+			t = strchr (folder, '/');
+			if (t) {
+				char *share = g_strndup (folder, t - folder);
+				gtk_entry_set_text (GTK_ENTRY (dialog->details->share_entry), share);
+				g_free (share);
+				folder = t + 1;
+			}
+			
+		}
+
+		/* Port */
+		if (meth->flags & SHOW_PORT) {
+			guint port = gnome_vfs_uri_get_host_port (uri);
+			if (port != 0) {
+				char sport[32];
+				g_snprintf (sport, sizeof (sport), "%d", port);
+				gtk_entry_set_text (GTK_ENTRY (dialog->details->port_entry), sport);
+			}
+		}
+
+		/* Folder */
+		gtk_entry_set_text (GTK_ENTRY (dialog->details->folder_entry), folder);
+		g_free (folder);
+
+		/* User */
+		if (meth->flags & SHOW_USER) {
+			const char *user = gnome_vfs_uri_get_user_name (uri);
+			if (user) {
+				t = strchr (user, ';');
+				if (t) {
+					user = t + 1;
+				}
+				gtk_entry_set_text (GTK_ENTRY (dialog->details->user_entry), user);
+			}
+		}
+
+		/* Domain */
+		if (meth->flags & SHOW_DOMAIN) {
+			const char *user = gnome_vfs_uri_get_user_name (uri);
+			if (user) {
+				t = strchr (user, ';');
+				if (t) {
+					char *domain = g_strndup (user, t - user);
+					gtk_entry_set_text (GTK_ENTRY (dialog->details->domain_entry), domain);
+					g_free (domain);
+				}
+			}
+		}
+	}
+#endif
+}
+
 static void
 combo_changed_callback (GtkComboBox *combo_box,
 			NautilusConnectServerDialog *dialog)
@@ -619,24 +750,37 @@ combo_changed_callback (GtkComboBox *combo_box,
 	setup_for_type (dialog);
 }
 
-
 static void
 port_insert_text (GtkEditable *editable,
 		  const gchar *new_text,
 		  gint         new_text_length,
 		  gint        *position)
 {
+	int pos;
+
 	if (new_text_length < 0) {
 		new_text_length = strlen (new_text);
 	}
 
-	if (new_text_length != 1 ||
-	    !g_ascii_isdigit (new_text[0])) {
-		gdk_display_beep (gtk_widget_get_display (GTK_WIDGET (editable)));
-		g_signal_stop_emission_by_name (editable, "insert_text");
+	/* Only allow digits to be inserted as port number */
+	for (pos = 0; pos < new_text_length; pos++) {
+		if (!g_ascii_isdigit (new_text[pos])) {
+			GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (editable));
+			if (toplevel != NULL) {
+				gdk_window_beep (toplevel->window);
+			}
+		    g_signal_stop_emission_by_name (editable, "insert_text");
+		    return;
+		}
 	}
 }
 
+static void
+bookmark_checkmark_toggled (GtkToggleButton *toggle, NautilusConnectServerDialog *dialog)
+{
+	gtk_widget_set_sensitive (GTK_WIDGET(dialog->details->name_entry),
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (toggle)));
+}
 
 static void
 nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
@@ -646,6 +790,9 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 	GtkWidget *combo;
 	GtkWidget *hbox;
 	GtkWidget *vbox;
+	GtkListStore *store;
+	GtkCellRenderer *renderer;
+	int i;
 	
 	dialog->details = g_new0 (NautilusConnectServerDialogDetails, 1);
 
@@ -672,23 +819,58 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    label, FALSE, FALSE, 0);
 
-	dialog->details->type_combo = combo = gtk_combo_box_new_text ();
-	/* Keep this in sync with enum */
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("SSH"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("Public FTP"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("FTP (with login)"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("Windows share"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("WebDAV (HTTP)"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("Secure WebDAV (HTTPS)"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-				   _("Custom Location"));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), TYPE_ANON_FTP);
+	dialog->details->type_combo = combo = gtk_combo_box_new ();
+
+	/* each row contains: method index, textual description */
+	store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
+	g_object_unref (G_OBJECT (store));
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
+	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), renderer, "text", 1);
+
+	for (i = 0; i < G_N_ELEMENTS (methods); i++) {
+		GtkTreeIter iter;
+		const gchar * const *supported;
+		int j;
+
+		/* skip methods that don't have corresponding GnomeVFSMethods */
+		supported = g_vfs_get_supported_uri_schemes (g_vfs_get_default ());
+
+		if (methods[i].scheme != NULL) {
+			gboolean found;
+
+			found = FALSE;
+			for (j = 0; supported[j] != NULL; j++) {
+				if (strcmp (methods[i].scheme, supported[j]) == 0) {
+					found = TRUE;
+					break;
+				}
+			}
+
+			if (!found) {
+				continue;
+			}
+		}
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter,
+				    0, i,
+				    1, get_method_description (&(methods[i])),
+				    -1);
+
+
+		if (methods[i].flags & DEFAULT_METHOD) {
+			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
+		}
+	}
+
+	if (gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) < 0) {
+		/* default method not available, use any other */
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
+	}
+
 	gtk_widget_show (combo);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
 	gtk_box_pack_start (GTK_BOX (hbox),
@@ -725,7 +907,24 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 	dialog->details->folder_entry = gtk_entry_new ();
 	dialog->details->domain_entry = gtk_entry_new ();
 	dialog->details->user_entry = gtk_entry_new ();
+	dialog->details->bookmark_check = gtk_check_button_new_with_mnemonic (_("Add _bookmark"));
 	dialog->details->name_entry = gtk_entry_new ();
+
+	g_signal_connect (dialog->details->bookmark_check, "toggled", 
+			  G_CALLBACK (bookmark_checkmark_toggled), dialog);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->details->bookmark_check), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET(dialog->details->name_entry), FALSE);
+
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->uri_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->server_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->share_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->port_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->folder_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->domain_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->user_entry), TRUE);
+	gtk_entry_set_activates_default (GTK_ENTRY (dialog->details->name_entry), TRUE);
+
 	/* We need an extra ref so we can remove them from the table */
 	g_object_ref (dialog->details->uri_entry);
 	g_object_ref (dialog->details->server_entry);
@@ -734,13 +933,14 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 	g_object_ref (dialog->details->folder_entry);
 	g_object_ref (dialog->details->domain_entry);
 	g_object_ref (dialog->details->user_entry);
+	g_object_ref (dialog->details->bookmark_check);
 	g_object_ref (dialog->details->name_entry);
 	
 	setup_for_type (dialog);
 	
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       _("Browse _Network"),
-			       RESPONSE_BROWSE);
+        gtk_dialog_add_button (GTK_DIALOG (dialog),
+                               GTK_STOCK_HELP,
+                               GTK_RESPONSE_HELP);
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       GTK_STOCK_CANCEL,
 			       GTK_RESPONSE_CANCEL);
@@ -758,16 +958,25 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 }
 
 GtkWidget *
-nautilus_connect_server_dialog_new (NautilusWindow *window)
+nautilus_connect_server_dialog_new (NautilusWindow *window, GFile *location)
 {
+	NautilusConnectServerDialog *conndlg;
 	GtkWidget *dialog;
 
 	dialog = gtk_widget_new (NAUTILUS_TYPE_CONNECT_SERVER_DIALOG, NULL);
+	conndlg = NAUTILUS_CONNECT_SERVER_DIALOG (dialog);
 
 	if (window) {
 		gtk_window_set_screen (GTK_WINDOW (dialog),
 				       gtk_window_get_screen (GTK_WINDOW (window)));
-		NAUTILUS_CONNECT_SERVER_DIALOG (dialog)->details->application = window->application;
+		conndlg->details->application = window->application;
+	}
+
+	if (location) {
+		/* If it's a remote URI, then load as the default */
+		if (!g_file_is_native (location)) {
+			display_server_location (conndlg, location);
+		}
 	}
 
 	return dialog;
